package ch.zhaw.ads;

import java.util.*;

public class MyHashtable<K,V> implements java.util.Map<K,V> {
    private K[] keys =   (K[]) new Object[10];
    private V[] values = (V[]) new Object[10];

    private int hash(Object k) {
        int h = Math.abs(k.hashCode()) % keys.length;
        int counter = 0;

        while (keys[h] != null && !keys[h].equals(k)) {
            h = (h + 1) % keys.length;
            counter++;
            if (counter >= keys.length) return -1;
        }
        return h;
    }

    public MyHashtable(int size) {
        keys =   (K[]) new Object[size];
        values = (V[]) new Object[size];
    }

    //  Removes all mappings from this map (optional operation).
    public void clear() {
        keys =   (K[]) new Object[keys.length];
        values = (V[]) new Object[values.length];
    }

    //  Associates the specified value with the specified key in this map (optional operation).
    public V put(K key, V value) {
        if (size() >= keys.length) {
            keys = Arrays.copyOf(keys, keys.length * 2);
            values = Arrays.copyOf(values, values.length * 2);
            rehash();
        }

        int keyId = hash(key);
        if (keyId < 0) return null;

        values[keyId] = value;
        keys[keyId] = key;

        return value;
    }

    //  Returns the value to which this map maps the specified key.
    public V get(Object key) {
        if (key == null) return null;
        return values[hash((K) key)];
    }

    //  Returns true if this map contains no key-value mappings.
    public boolean isEmpty() {
        return size() == 0;
    }

    //  Removes the mapping for this key from this map if present (optional operation).
    public V remove(Object key) {
        int keyId = hash((K) key);
        if (keyId < 0) return null;

        V removed = values[keyId];
        values[keyId] = null;
        keys[keyId] = null;
        rehash();
        return removed;
    }

    //  Returns the number of key-value mappings in this map.
    public int size() {
        int size = 0;
        for (int i = 0; i < keys.length; i++) {
            if (keys[i] != null) size++;
        }
        return size;
    }

    private void rehash() {
        K[] keysClone = keys.clone();
        V[] valuesClone = values.clone();
        clear();

        for (int i = 0; i < keys.length; i++) {
            if (keysClone[i] != null) {
                put(keysClone[i], valuesClone[i]);
            }
        }
    }

    // =======================================================================
    //  Returns a set view of the keys contained in this map.
    public Set keySet() {
        throw new UnsupportedOperationException();
    }

    //  Copies all of the mappings from the specified map to this map (optional operation).
    public void putAll(Map t) {
        throw new UnsupportedOperationException();
    }

    //  Returns a collection view of the values contained in this map.
    public Collection values() {
        throw new UnsupportedOperationException();
    }
    //  Returns true if this map contains a mapping for the specified key.
    public boolean containsKey(Object key) {
        throw new UnsupportedOperationException();
    }
    //  Returns true if this map maps one or more keys to the specified value.
    public boolean containsValue(Object value)  {
        throw new UnsupportedOperationException();
    }
    //  Returns a set view of the mappings contained in this map.
    public Set entrySet() {
        throw new UnsupportedOperationException();
    }
    //  Compares the specified object with this map for equality.
    public boolean equals(Object o) {
        throw new UnsupportedOperationException();
    }
    //  Returns the hash code value for this map.
    public int hashCode() {
        throw new UnsupportedOperationException();
    }

}
